
__all__ = ['INVERT', 'BOR', 'BXOR', 'BAND', 'LSHIFT', 'RSHIFT']

from .base import Head, UnaryHead, NaryHead

class InvertHead(UnaryHead):
    """
    InvertHead is unary invert operation, data is expression operand.
    """
    op_mth = '__invert__'
    op_symbol = '~'
    def __repr__(self): return 'INVERT'

class BorHead(NaryHead):
    """
    BorHead is a head of n-ary bitwise inclusive (normal) OR operation,
    data is n-tuple of expression operands.
    """
    op_mth = '__or__'
    op_symbol = '|'
    def __repr__(self): return 'BOR'

class BxorHead(NaryHead):
    """
    BxorHead is a head of n-ary bitwise exclusive OR operation,
    data is n-tuple of expression operands.
    """
    op_mth = '__xor__'
    op_symbol = '^'
    def __repr__(self): return 'BXOR'

class BandHead(NaryHead):
    """
    BandHead is a head of n-ary bitwise AND operation,
    data is n-tuple of expression operands.
    """
    op_mth = '__and__'
    op_symbol = '&'
    def __repr__(self): return 'BAND'

class LshiftHead(NaryHead):
    """
    LshiftHead is a head of n-ary bitwise shift left operation,
    data is n-tuple of expression operands.
    """
    op_mth = '__lshift__'
    op_symbol = '<<'
    def __repr__(self): return 'LSHIFT'

class RshiftHead(NaryHead):
    """
    RshiftHead is a head of n-ary bitwise shift right operation,
    data is n-tuple of expression operands.
    """
    op_mth = '__rshift__'
    op_symbol = '>>'
    def __repr__(self): return 'RSHIFT'

INVERT = InvertHead()
BOR = BorHead()
BXOR = BxorHead()
BAND = BandHead()
LSHIFT = LshiftHead()
RSHIFT = RshiftHead()



from sympycore.calculus import *
A = Calculus.convert

x = Symbol('x')
y = Symbol('y')

def test_constructor():
    assert repr(A(1.2))=="Calculus('1.2')"
    assert repr(A('1.2'))=="Calculus('1.2')"

def test_linear():
    assert x + x == 2*x
    assert (2*x) / 2 == x
    assert (x/2)*2 == x
    assert 3*x + 4*x == 7*x
    assert 2*x/3 + x/3 == x
    assert I*x + 3*x == (3+I)*x

def test_powers():
    assert Number(3) ** Number(-2) == Number(1,9)
    assert Number(4) ** Number(1,2) == Number(2)
    assert 2**Number(3,2) * 2**Number(1,2) == 4, `2**Number(3,2) * 2**Number(1,2)`
    assert I**Number(3,2) * I**Number(1,2) == -1
    assert str(Number(2) ** Number(1,2)) == '2**(1/2)', str(Number(2) ** Number(1,2))
    assert A('4/9') ** A('1/2') == A('2/3')
    assert A('4/7') ** A('1/2') == A('7**(-1/2)')*2
    assert A('7/4') ** A('1/2') == A('7**(1/2)')/2
    assert str((I*x)**2)==str('-x**2')
    assert str((-I*x)**2)==str('-x**2')
    assert str((I*x)**4)==str('x**4')


def test_has_symbol():
    assert (1 + Cos(1+2**x)).has_symbol(x)
    assert (y + Cos(1+2**x)).symbols == set([x, y])

def test_subs():
    assert (oo*x + oo*y).subs(y,x) == oo*x
    assert (oo*x + oo*y).subs(y,-x) == undefined

def test_diff():
    x = Symbol('x')
    assert Number(2).diff(x) == 0
    assert x.diff(x) == 1
    assert (3*x).diff(x) == 3
    assert (3*x+1).diff(x) == 3
    assert (2*x**2 + x).diff(x) == 4*x + 1
    assert (x**3).diff(x) == 3*x**2
    assert ((1+x)*(2+x)*(3+x)**2).diff(x) in [((1+x)*(2+x)*(3+x)*2 + (1+x)*(3+x)**2 + (2+x)*(3+x)**2),
                                              (1 + x)*(2 + x)*(6 + 2*x) + (3 + x)**2*(1 + x) + (3 + x)**2*(2 + x)]

def test_subs():
    x = Symbol('x')
    y = Symbol('y')
    assert 2*x | (x, pi) == 2*pi
    assert x*y | {x:2, y:pi} == 2*pi



#!/usr/bin/env python

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('dsge', parent_package, top_path)
    config.add_subpackage('solvers')
    config.add_subpackage('parsers')
    config.add_subpackage('updaters')
    return config

if __name__ == '__main__':
    print('This is the wrong setup.py file to run')





from .base import Head, heads_precedence

class IntegralIndefiniteHead(Head):

    def __repr__(self): return 'INTEGRAL_INDEFINITE'

    def data_to_str_and_precedence(self, cls, (integrand, symbol)):
        o_p = heads_precedence.APPLY
        s, s_p = integrand.head.data_to_str_and_precedence(cls, integrand.data)
        return 'Integral(%s, %s)' % (s, symbol), o_p


INTEGRAL_INDEFINITE = IntegralIndefiniteHead()
    



""" Implements matrix determinant algoritms.
"""
# Author: Pearu Peterson
# Created: March 2008
from ..arithmetic.numbers import div
from .linalg import swap_rows_MATRIX

def MATRIX_DICT_determinant(self, overwrite=False):
    """ Determinant of a n x n matrix.

    Using fraction-free Gaussian elimination method when n>3.
    For n<=3, direct expressions are used.
    """
    head, data = self.pair

    m, n = head.shape
    assert m==n,`m,n`
    if head.is_diagonal:
        dget = data.get
        d = 1
        for i in range (m):
            v = dget((i,i))
            if v is None:
                return 0
            d *= v
        return d
    if not (overwrite and self.is_writable):
        data = dict(data)

    return determinant_MATRIX(m, data)

def get_minor_MATRIX(p, q, data):
    d = {}
    for (i,j),x in data.items():
        if p==i or q==j:
            continue
        if i<p:
            if j<q:
                d[i,j] = x
            else:
                d[i,j-1] = x
        else:
            if j<q:
                d[i-1,j] = x
            else:
                d[i-1,j-1] = x
    return d

def _det2(a_00, a_01, a_10, a_11):
    if a_00 and a_11:
        if a_01 and a_10:
            return a_00*a_11 - a_01*a_10
        return a_00*a_11
    if a_01 and a_10:
        return -a_01*a_10
    return 0

def _det3(a_00, a_01, a_02, a_10, a_11, a_12, a_20, a_21, a_22):
    if a_00:
        if a_01:
            if a_02:
                return a_00 * _det2(a_11, a_12, a_21, a_22) \
                       - a_01 * _det2(a_10, a_12, a_20, a_22) \
                       + a_02 * _det2(a_10, a_11, a_20, a_21)
            return a_00 * _det2(a_11, a_12, a_21, a_22) \
                   - a_01 * _det2(a_10, a_12, a_20, a_22)
        if a_02:
            return a_00 * _det2(a_11, a_12, a_21, a_22) \
                   + a_02 * _det2(a_10, a_11, a_20, a_21)
        return a_00 * _det2(a_11, a_12, a_21, a_22)
    if a_01:
        if a_02:
            return - a_01 * _det2(a_10, a_12, a_20, a_22) \
                   + a_02 * _det2(a_10, a_11, a_20, a_21)
        return - a_01 * _det2(a_10, a_12, a_20, a_22)
    if a_02:
        return + a_02 * _det2(a_10, a_11, a_20, a_21)
    return 0

def determinant_MATRIX(m, data):
    data_get = data.get
    if m==1:
        return data_get((0,0),0)
    if m==2:
        a_00 = data_get((0,0))
        a_01 = data_get((0,1))
        a_10 = data_get((1,0))
        a_11 = data_get((1,1))
        return _det2(a_00, a_01, a_10, a_11)
    if m==3:
        a_00 = data_get((0,0))
        a_01 = data_get((0,1))
        a_02 = data_get((0,2))
        a_10 = data_get((1,0))
        a_11 = data_get((1,1))
        a_12 = data_get((1,2))
        a_20 = data_get((2,0))
        a_21 = data_get((2,1))
        a_22 = data_get((2,2))
        return _det3(a_00, a_01, a_02, a_10, a_11, a_12, a_20, a_21, a_22)
    
    newdata = {}
    d = 1
    sign = 1
    for k in range(m-1):
        a_kk = data_get((k,k))
        k1 = k+1
        if not a_kk:
            for i in range(k1,m):
                a_kk = data_get((i,k))
                if a_kk:
                    break
            if not a_kk:
                return 0
            sign = -sign
            swap_rows_MATRIX(data, i, k)
        for i in range(k1,m):
            a_ik = data_get((i,k))
            if a_ik:
                for j in range(k1, m):
                    ij = i,j
                    a_ij = data_get(ij)
                    if a_ij:
                        a_kj = data_get((k,j))
                        if a_kj:
                            x = a_kk * a_ij - a_ik * a_kj
                            if x:
                                if d is 1:
                                    newdata[ij] = x
                                else:
                                    newdata[ij] = div(x, d)
                        else:
                            if d is 1:
                                newdata[ij] = a_kk * a_ij
                            else:
                                newdata[ij] = div(a_kk * a_ij, d)
                    else:
                        a_kj = data_get((k,j))
                        if a_kj:
                            if d is 1:
                                newdata[ij] = -a_ik * a_kj
                            else:
                                newdata[ij] = div(-a_ik * a_kj, d)
            else:
                for j in range(k1, m):
                    ij = i,j
                    a_ij = data_get(ij)
                    if a_ij:
                        if d is 1:
                            newdata[ij] = a_kk * a_ij
                        else:
                            newdata[ij] = div(a_kk * a_ij, d)
        d = a_kk
        data = newdata
        data_get = data.get
        newdata = {}
    return sign * data.get((m-1,m-1),0)



import os
import pymaclab as pm
from pymaclab import __path__ as root_dir
root_dir = root_dir[0]


# Now do for stable branch
def expose_stable(scopedic=None):   
	fcont = os.listdir(os.path.join(root_dir,'modfiles/models/stable'))
	modfiles = [x for x in fcont if x[-4:] == '.txt']
	modpdic = {}
	for modo in modfiles:
		modpdic[modo.split('.')[0]] = os.path.join(root_dir,'modfiles/models/stable',modo)
	# Now expose the model files' paths to the local scope to make it accessible for users
	for elem in modpdic.keys():
		exec elem+'='+'"'+modpdic[elem]+'"' in scopedic

# Now do for development branch
def expose_development(scopedic=None):
	fcont = os.listdir(os.path.join(root_dir,'modfiles/models/development'))
	modfiles = [x for x in fcont if x[-4:] == '.txt']
	modpdic = {}
	for modo in modfiles:
		modpdic[modo.split('.')[0]] = os.path.join(root_dir,'modfiles/models/development',modo)
	# Now expose the model files' paths to the local scope to make it accessible for users
	for elem in modpdic.keys():
		exec elem+'='+'"'+modpdic[elem]+'"' in scopedic

# Now do for testing branch
def expose_testing(scopedic=None):
	fcont = os.listdir(os.path.join(root_dir,'modfiles/models/testing'))
	modfiles = [x for x in fcont if x[-4:] == '.txt']
	modpdic = {}
	for modo in modfiles:
		modpdic[modo.split('.')[0]] = os.path.join(root_dir,'modfiles/models/testing',modo)
	# Now expose the model files' paths to the local scope to make it accessible for users
	for elem in modpdic.keys():
		exec elem+'='+'"'+modpdic[elem]+'"' in scopedic


# Now delete the objects we no longer want in this namespace
def delete_all():
	if 'os' in dir(): del os
	if 'root_dir' in dir(): del root_dir
	if 'fcont' in dir(): del fcont
	if 'modfiles' in dir(): del modfiles
	if 'modpdic' in dir(): del modpdic
	if 'x' in dir(): del x
	if 'elem' in dir(): del elem
	if 'modo' in dir(): del modo
	if 'pm' in dir(): del pm




__all__ = ['RingInterface']

from ..core import init_module

@init_module
def _init(m):
    from ..core import heads
    for n,h in heads.iterNameValue(): setattr(m, n, h)

class RingInterface:

    zero = 0
    one = 1

    @classmethod
    def Add(cls, *seq):
        """ Construct a symbolic expression with addition operands
        list representation.
        """
        return ADD.reevaluate(cls, seq)

    @classmethod
    def Sub(cls, *seq):
        return SUB.reevaluate(cls, seq)

    @classmethod
    def Mul(cls, *seq):
        """ Construct a symbolic expression with multiplication
        operands list representation.
        """
        return MUL.reevaluate(cls, seq)

    @classmethod
    def Div(cls, *seq):
        return DIV.reevaluate(cls, seq)

    @classmethod
    def Pow(cls, base, exp):
        return POW.new(cls, (base, exp))

    @classmethod
    def Terms(cls, *seq):
        """ Construct a symbolic expression with term-coefficient
        representation.
        """
        return TERM_COEFF_DICT.reevaluate(cls, dict(seq))

    @classmethod
    def Factors(cls, *seq):
        """ Construct a symbolic expression with base-exponent
        representation.
        """
        return BASE_EXP_DICT.reevaluate(cls, dict(seq))

    @classmethod
    def Polynom(cls, *seq, **kws):
        """ Construct a symbolic expression with exponent-coefficient
        representation:

          Polynom(x,y) -> Algebra(EXP_COEFF_DICT, Pair((x,y), {(1,0):1, (0,1):1}))
          Polynom(x,y, variables=(x,y,z)) -> Algebra(EXP_COEFF_DICT, Pair((x,y,z), {(1,0,0):1, (0,1,0):1}))
        """
        r = cls.Add(*seq)
        return r.head.to_EXP_COEFF_DICT(cls, r.data, r, kws.get('variables'))




from ..core import classes
from .algebra import Logic

def Eq(a, b):
    """ Return relational expression ``a==b``.
    """
    c = classes.Calculus.convert
    return Logic.Eq(c(a), c(b))

def Ne(a, b):
    """ Return relational expression ``a!=b``.
    """
    c = classes.Calculus.convert
    return Logic.Ne(c(a), c(b))

def Lt(a, b):
    """ Return relational expression ``a<b``.
    """
    c = classes.Calculus.convert
    return Logic.Lt(c(a), c(b))

def Le(a, b):
    """ Return relational expression ``a<=b``.
    """
    c = classes.Calculus.convert
    return Logic.Le(c(a), c(b))

def Gt(a, b):
    """ Return relational expression ``a>b``.
    """
    c = classes.Calculus.convert
    return Logic.Gt(c(a), c(b))

def Ge(a, b):
    """ Return relational expression ``a>=b``.
    """
    c = classes.Calculus.convert
    return Logic.Ge(c(a), c(b))

def IsElement(a, b):
    """ Return relational expression ``a in b``.
    """
    c = classes.Calculus.convert
    #XXX: b = classes.Set.convert(b)
    return Logic.IsElement(c(a), b)

def Not(a):
    """ Return boolean expression ``not a``.
    """
    return Logic.Not(Logic.convert(a))

def And(*seq):
    """ Return boolean expression ``x1 and x2 and ...``.
    """
    c = Logic.convert
    return Logic.And(*map(c, seq))

def Or(*seq):
    """ Return boolean expression ``x1 or x2 or ...``.
    """
    c = Logic.convert
    return Logic.Or(*map(c, seq))




from sympycore import CollectingField as Algebra

Symbol = Algebra.Symbol
Number = Algebra.Number
Add = Algebra.Add
Mul = Algebra.Mul
Pow = Algebra.Pow
Terms = Algebra.Terms
Factors = Algebra.Factors

x,y,z = map(Symbol, 'xyz')

def test_simple():
    assert x.subs(x,y)==y
    assert x.subs(z,y)==x
    assert (2*x).subs(x,y)==2*y
    assert (x+z).subs(x,y)==y+z
    assert (x**2).subs(x,y)==y**2
    assert (x*z).subs(x,y)==y*z
    assert (x**2*z).subs(x,y)==y**2*z
    assert (x + x*z).subs(x,y)==y+y*z
    
def test_repeated():
    assert x.subs([(x,y)])==y
    assert (x + x*z).subs([(x,y),(z,x)])==y+y*x,`(x + x*z).subs([(x,y),(z,x)])`
    assert (x+y).subs({x:1,y:2*z})==1+2*z,`((x+y).subs({x:1,y:2*z})).pair, (1+2*z).pair`




from .base import AtomicHead, heads_precedence, Expr

import re
_is_atomic = re.compile(r'\A\w+\Z').match

class SpecialHead(AtomicHead):
    """
    SpecialHead is head for special objects like None, NotImplemented,
    Ellipsis, etc.  Data can be any Python object.
    """

    def __repr__(self): return 'SPECIAL'

    def data_to_str_and_precedence(self, cls, data):
        if isinstance(data, Expr):
            h, d = data.pair
            return h.data_to_str_and_precedence(cls, d)
        if data is Ellipsis:
            return '...', heads_precedence.SYMBOL

        s = str(data)
        if _is_atomic(s):
            return s, heads_precedence.SYMBOL
        return s, 0.0 # force parenthesis

SPECIAL = SpecialHead()



""" Provides Unit class.
"""

__docformat__ = "restructuredtext"
__all__ = [\
    'Unit', 'meter', 'kilogram', 'second', 'ten', 'yotta', 'zetta',
    'exa', 'peta', 'tera', 'giga', 'mega', 'kilo', 'deca', 'deci',
    'centi', 'milli', 'micro', 'nano', 'pico', 'femto', 'atto',
    'zepto', 'yocto' ]

from ..core import classes

from ..calculus.algebra import Calculus, algebra_numbers
from ..ring import CommutativeRing

class Unit(CommutativeRing):
    """ Represents an algebra of physical units.

    Elements of the units algebra are unit symbols.
    Coefficients of the units algebra are Calculus instances.
    """

    def as_algebra(self, cls, typeerror=True):
        """ Convert algebra to another algebra.
        """
        if cls is classes.Verbatim:
            return self.as_verbatim()
        if cls is Calculus:
            return NotImplemented
        return self.as_verbatim().as_algebra(cls)

classes.Unit = Unit

meter = Unit.Symbol('m')
kilogram = Unit.Symbol('kg')
second = Unit.Symbol('s')

def init_module(m):


    m.ten = ten = Calculus.Number(10)
    m.yotta = ten**24
    m.zetta = ten**21
    m.exa   = ten**18
    m.peta  = ten**15
    m.tera  = ten**12
    m.giga  = ten**9
    m.mega  = ten**6
    m.kilo  = ten**3
    m.deca  = ten**1
    m.deci  = ten**-1
    m.centi = ten**-2
    m.milli = ten**-3
    m.micro = ten**-6
    m.nano  = ten**-9
    m.pico  = ten**-12
    m.femto = ten**-15
    m.atto  = ten**-18
    m.zepto = ten**-21
    m.yocto = ten**-24




#
# Created in 2007 by Fredrik Johansson
# Expression parser added by Pearu Peterson
#
""" Provides PrimitiveAlgebra class and expression parser.
"""
from __future__ import absolute_import
__docformat__ = "restructuredtext"
__all__ = ['Verbatim']

import types
import compiler
from compiler import ast

from .algebra import Algebra
from ..heads import (OR, AND, NOT, LT, LE, GT, GE, EQ, NE, BAND, BOR, BXOR,
                     INVERT, POS, NEG, ADD, SUB, MOD, MUL, DIV, FLOORDIV, POW,
                     LSHIFT, RSHIFT, IS, ISNOT, LIST, SLICE,
                     NUMBER, SYMBOL, APPLY, TUPLE, LAMBDA, TERMS, BASE_EXP_DICT,
                     IN, NOTIN, SUBSCRIPT, SPECIAL, DICT, ATTR, KWARG)
from ..heads import CALLABLE
from ..core import classes, Expr, objects

# Restrictions:
#
#    Star and double star function arguments are not implemented,
#    i.e. parsing 'f(*a)' and 'f(**b)' will fail.
#
# TODO: IfExp support: parse `a if b else c` to Verbatim(IF, (b, a, c))

EllipsisType = type(Ellipsis)
special_types = (EllipsisType, type(None), type(NotImplemented))
special_objects = set([Ellipsis, None, NotImplemented])
number_types = (int, long, float, complex)

containing_lst = set([IN, NOTIN])

atomic_lst = set([SYMBOL, NUMBER])
unary_lst = set([POS, NEG, NOT, INVERT])
binary_lst = set([AND, OR, BAND, BOR, BXOR, ADD, SUB, MUL, DIV, FLOORDIV,
                  MOD, POW, LSHIFT, RSHIFT])

convert_head_Op_map = {
    NEG : 'Neg',
    POS : 'Pos',
    ADD : 'Add',
    SUB : 'Sub',
    MUL : 'Mul',
    DIV : 'Div',
    FLOORDIV : 'FloorDiv',
    POW : 'Pow',
    MOD : 'Mod',
    AND : 'And',
    OR : 'Or',
    NOT : 'Not',
    LT : 'Lt',
    GT : 'Gt',
    LE : 'Le',
    GE : 'Ge',
    EQ : 'Eq',
    NE : 'Ne',
    }

class Verbatim(Algebra):
    """ Represents an unevaluated expression.
    """

    commutative_add = None
    commutative_mul = None
    disable_sorting = None

    _str = None

    @classmethod
    def get_value_algebra(cls):
        return cls

    def get_argument_algebra(self, index):
        return self.get_value_algebra()

    @classmethod
    def get_function_algebra(cls):
        return cls

    @classmethod
    def get_differential_algebra(cls):
        return cls
    
    @classmethod
    def convert(cls, obj):
        if isinstance(obj, (str, unicode)):
            obj = string2Verbatim(obj)
        if isinstance(obj, Verbatim):
            return obj
        if hasattr(obj, 'as_verbatim'):
            # handle low-level numbers and constants, as well as Verbatim subclasses
            try:
                return obj.as_verbatim()
            except TypeError, msg:
                if str(msg)=='unbound method as_verbatim() must be called with Verbatim instance as first argument (got nothing instead)':
                    pass
                else:
                    raise
        if isinstance(obj, slice):
            slice_args = obj.start, obj.stop, obj.step
            return cls(SLICE, tuple(map(cls.convert, slice_args)))
        elif isinstance(obj, tuple):
            return cls(TUPLE, tuple(map(cls.convert, obj)))
        elif isinstance(obj, list):
            return cls(LIST, tuple(map(cls.convert, obj)))
        elif isinstance(obj, special_types):
            return cls(SPECIAL, obj)
        elif isinstance(obj, number_types):
            return cls(NUMBER, obj)
        elif isinstance(obj, dict):
            return cls(DICT,  tuple([(cls.convert(k), cls.convert(v)) for k,v in obj.iteritems()]))
        return Verbatim(SYMBOL, obj)

    def as_verbatim(self):
        return self

    def as_algebra(self, cls, source=None):
        def as_algebra(cls, head, data, target):
            if head is NUMBER:
                return cls(data)
            if head is SYMBOL:
                return cls.convert_symbol(data)
            return head.reevaluate(cls, data)
        head, rest = self.pair
        return head.walk(as_algebra, cls, rest, self)

    def __repr__(self):
        return '%s(%r, %r)' % (type(self).__name__, self.head, self.data)

    def as_tree(self, tab='', level=0):
        if level:
            r = []
        else:
            r = [self.__class__.__name__+':']
        head, rest = self.pair
        if head in atomic_lst:
            r.append(tab + '%r[%s]' % (head, rest))
        else:
            r.append(tab + '%r[' % (head))
            if isinstance(rest, Verbatim):
                rest = rest,
            for t in rest:
                if type(t) is tuple:
                    r.append(tab + '  (')
                    for t1 in t:
                        r.append(t1.as_tree(tab=tab + '    ', level=level+1))
                    r.append(tab + '  )')
                else:
                    r.append(t.as_tree(tab=tab + '  ', level=level+1))

            r.append(tab+']')
        return '\n'.join(r)

    def __eq__(self, other):
        if type(other) is Verbatim:
            return self.pair == other.pair
        return False

    for _h in unary_lst:
        if _h.op_mth:
            exec '''\
def %s(self):
    return Verbatim(%r, self)
''' % (_h.op_mth, _h)

    for _h in binary_lst:
        if _h.op_mth:
            exec '''\
def %s(self, other):
    other = self.convert(other)
    return Verbatim(%r, (self, other))
''' % (_h.op_mth, _h)
        if _h.op_rmth:
            exec '''\
def %s(self, other):
    other = self.convert(other)
    return Verbatim(%r, (other, self))
''' % (_h.op_rmth, _h)

    __truediv__ = __div__
    __rtruediv__ = __rdiv__

    def __pow__(self, other):
        other = self.convert(other)
        if other.head is NUMBER:
            other = other.data
        return Verbatim(POW, (self, other))

    def __rpow__(self, other):
        other = self.convert(other)
        if self.head is NUMBER:
            self = self.data
        return Verbatim(POW, (other, self))


    def __divmod__(self, other):
        other = self.convert(other)
        return Verbatim(APPLY, (Verbatim(CALLABLE, divmod), (self, other)))

    def __call__(self, *args, **kwargs):
        convert = self.convert
        args = map(convert, args)
        for k, v in kwargs.items():
            args.append(Verbatim(KWARG, (convert(k), convert(v))))
        return Verbatim(APPLY, (self, tuple(args)))

    def __getitem__(self, key):
        if type(key) is tuple:
            key = tuple(map(self.convert, key))
            return Verbatim(SUBSCRIPT, (self, key))
        key = self.convert(key)
        return Verbatim(SUBSCRIPT, (self, (key,)))

    def __getattr__(self, attr):
        # warning: with this feature hasattr() may return True when not desired.
        if not attr.startswith('_'):
            return Verbatim(ATTR, (self, self.convert(attr)))
        raise AttributeError

classes.Verbatim = Verbatim

########### string to Verbatim parser ############

node_names = []
skip_names = ['Module','Stmt','Discard']
for n, cls in ast.__dict__.items():
    if n in skip_names:
        continue
    if isinstance(cls, (type,types.ClassType)) and issubclass(cls, ast.Node):
        node_names.append(n)

node_map = dict(Add=ADD, Mul=MUL, Sub=SUB,
                Div=DIV, FloorDiv=FLOORDIV, TrueDiv=DIV,
                UnaryAdd=POS, UnarySub=NEG, Mod=MOD, Not=NOT,
                Or=OR, And=AND, Power=POW,
                Bitand=BAND,Bitor=BOR,Bitxor=BXOR,
                Tuple=TUPLE, Subscript=SUBSCRIPT,
                Invert=INVERT, LeftShift=LSHIFT, RightShift=RSHIFT,
                List=LIST, Sliceobj=SLICE
                )

callfunc_map = dict(#divmod=DIVMOD,
                    slice=SLICE)

compare_map = {'<':LT, '>':GT, '<=':LE, '>=':GE,
               '==':EQ, '!=':NE, 'in':IN, 'not in': NOTIN,
               'is':IS, 'is not':ISNOT}

class VerbatimWalker:
    """ Helper class for expression parser.
    """

    def __init__(self):
        self.stack = []

    # for composite instance:
    def start(self, head):
        self.stack.append([head, []])
    def append(self, obj):
        stack = self.stack
        if not stack:
            stack.append(obj)
        else:
            stack[-1][1].append(obj)
    def end(self, tuple_head = None):
        head, lst = self.stack.pop()
        if self.stack:
            last = self.stack[-1]
            if last[0]==head and head in [ADD, MUL]:
                # apply associativity:
                last[1].extend(lst)
                return
        if head is tuple_head:
            self.append(tuple(lst))
        elif head in unary_lst:
            assert len(lst)==1,`lst`
            self.append(Verbatim(head, lst[0]))
        elif head is POW:
            b, e = lst
            if e.head is NUMBER:
                e = e.data
            self.append(Verbatim(head, (b, e)))
        else:
            self.append(Verbatim(head, tuple(lst)))
    
    # for atomic instance:
    def add(self, *args):
        self.append(Verbatim(*args))

    for _n in node_names:
        if _n in node_map:
            continue
        exec '''\
def visit%s(self, node, *args):
    print "WARNING: using default unsupported visit%s, node=", node
    self.start(%r)
    for child in node.asList():
        if isinstance(child, ast.Node):
            self.visit(child, *args)
        else:
            self.append(child)
    self.end()
''' % (_n, _n, _n)

    for _n,_v in node_map.items():
        exec '''\
def visit%s(self, node):
    self.start(%r)
    for child in node.getChildNodes():
        self.visit(child)
    self.end()
''' % (_n, _v)

    # visitNode methods:

    def visitSubscript(self, node):
        self.start(SUBSCRIPT)
        self.visit(node.expr)
        self.start(TUPLE)
        map(self.visit, node.subs)
        self.end(tuple_head = TUPLE)
        self.end()

    def visitName(self, node):
        self.add(SYMBOL, node.name)

    def visitConst(self, node):
        if node.value in special_objects:
            self.add(SPECIAL, node.value)
        else:
            self.add(NUMBER, node.value)

    def visitCompare(self, node):
        lhs = node.expr
        op, rhs = node.ops[0]
        if len(node.ops)==1:
            self.start(compare_map[op])
            self.visit(lhs)
            self.visit(rhs)
            self.end()
            return
        n = ast.And([ast.Compare(lhs, node.ops[:1]),
                     ast.Compare(rhs, node.ops[1:])])
        self.visit(n)

    def visitLambda(self, node):
        assert not (node.kwargs or node.varargs),`node.kwargs, node.varargs` # parsing `lambda *args, **kwargs: ..` not supported
        self.start(LAMBDA)
        self.start(TUPLE)
        for n,d in zip(node.argnames, (len(node.argnames) - len(node.defaults))*[None] + list(node.defaults)):
            if d is None:
                self.visit(ast.Name(n))
            else:
                self.visit(ast.Keyword(n, d))
        self.end(tuple_head=TUPLE)
        self.visit(node.code)
        self.end()

    def visitCallFunc(self, node):
        if node.star_args is not None:
            raise NotImplementedError('parsing function star arguments')
        if node.dstar_args is not None:
            raise NotImplementedError('parsing function double star arguments')
        func = node.node
        if isinstance(func, ast.Name) and func.name in callfunc_map:
            self.start(callfunc_map[func.name])
            for child in node.args:
                self.visit(child)
            self.end()
            return
        self.start(APPLY)
        self.visit(func)
        self.start(TUPLE)
        for child in node.args:
            self.visit(child)
        self.end(tuple_head = TUPLE)
        self.end()

    def visitSlice(self, node):
        n = ast.Subscript(node.expr, compiler.consts.OP_APPLY, [ast.Sliceobj(node.asList()[2:])])
        self.visit(n)

    def visitSliceobj(self, node):
        childs = list(node.asList())
        childs.extend([None]*(3-len(childs)))
        assert len(childs)==3,`childs`
        self.start(SLICE)
        for child in childs:
            if child is None:
                self.add(SPECIAL, child)
            else:
                self.visit(child)
        self.end()

    def visitEllipsis(self, node):
        self.add(SPECIAL, Ellipsis)

    def visitDict(self, node):
        self.start(DICT)
        for k,v in node.items:
            self.start(TUPLE)
            self.visit(k)
            self.visit(v)
            self.end(tuple_head = TUPLE)
            # collect key and value to a 2-tuple:
            #self.stack[-1][1][-2] = tuple(self.stack[-1][1][-2:])
            #del self.stack[-1][1][-1]
        self.end()

    def visitGetattr(self, node):
        self.start(ATTR)
        self.visit(node.expr)
        self.visit(ast.Name(node.attrname))
        self.end()

    def visitKeyword(self, node):
        self.start(KWARG)
        self.visit(ast.Name(node.name))
        self.visit(node.expr)
        self.end()

class ast_Pair(ast.Tuple):
    pass

def string2Verbatim(expr):
    """ Parse string expr to Verbatim.
    """
    node = compiler.parse(expr)
    return compiler.walk(node, VerbatimWalker()).stack.pop()



"""Provides various implementation specific constants (expression heads, etc).
"""

__docformat__ = 'restructuredtext'

import inspect

class HEAD(object):
    """ Base class to head constants.

    Head constants are singletons.
    """
    _cache = {}
    def __new__(cls, *args):
        key = '%s%s' % (cls.__name__, args)
        obj = cls._cache.get(key)
        if obj is None:
            obj = object.__new__(cls)
            cls._cache[key] = obj
            obj._key = key
            obj.init(*args)
        return obj

    def init(self, *args):
        # derived class may set attributes here
        pass #pragma NO COVER

    def as_unique_head(self):
        # used by the pickler support to make HEAD instances unique
        try:
            return self._cache[self._key]
        except KeyError:
            self._cache[self._key] = self
            return self

    def is_data_ok(self, cls, expr):
        return

# The following constants define both the order of operands
# as well as placing parenthesis for classes deriving from
# CollectingField:

str_SUM = -1
str_PRODUCT = -2
str_POWER = -3
str_APPLY = -4
str_SYMBOL = -5
str_NUMBER = -6

# The following constants are used by Verbatim and
# CollectingField classes.



#POLY = HEAD('POLY')
#DENSE_POLY = HEAD('DENSE_POLY')

DIFF = HEAD('DIFF')

MATRIX_DICT = intern('MATRIX_DICT')
MATRIX_DICT_T = intern('MATRIX_DICT_T')
MATRIX_DICT_A = intern('MATRIX_DICT_A')
MATRIX_DICT_TA = intern('MATRIX_DICT_TA')
MATRIX_DICT_D = intern('MATRIX_DICT_D')
MATRIX_DICT_TD = intern('MATRIX_DICT_TD')

class MATRIX(HEAD):
    """ Matrix head singleton class.

    Usage::

      MATRIX(<rows>, <cols>, <storage>)

    where
    
      ``<rows>``     - number of matrix rows
      ``<cols>``     - number of matrix columns
      ``<strorage>`` - constant describing data storage properties:
                       MATRIX_DICT, MATRIX_DICT_T, MATRIX_DICT_A, MATRIX_DICT_TA,
                       MATRIX_DICT_D, MATRIX_DICT_DT
    """

    def __str__ (self):
        return '%s(%r, %r, %r)' % (self.__class__.__name__, self.rows, self.cols, self.storage)

    __repr__ = __str__

    def to_lowlevel(self, cls, data, pair):
        return pair
    
    def init(self, rows, cols, storage):
        self.rows = rows
        self.cols = cols
        self.shape = (rows, cols)
        self.storage = storage

        self.is_transpose = is_transpose = storage in [MATRIX_DICT_T, MATRIX_DICT_TA, MATRIX_DICT_TD]
        self.is_array = storage in [MATRIX_DICT_A, MATRIX_DICT_TA]
        self.is_diagonal = storage in [MATRIX_DICT_D, MATRIX_DICT_TD]

        if storage==MATRIX_DICT:
            self.T = type(self)(cols, rows,  MATRIX_DICT_T)
            self.A = type(self)(rows, cols,  MATRIX_DICT_A)
            self.M = self
            self.D = type(self)(rows, cols,  MATRIX_DICT_D)
        elif storage==MATRIX_DICT_T:
            self.T = type(self)(cols, rows,  MATRIX_DICT)
            self.A = type(self)(rows, cols,  MATRIX_DICT_TA)
            self.M = self
            self.D = type(self)(rows, cols,  MATRIX_DICT_TD)
        elif storage==MATRIX_DICT_A:
            self.T = type(self)(cols, rows,  MATRIX_DICT_TA)
            self.A = self
            self.M = type(self)(rows, cols,  MATRIX_DICT)
            self.D = type(self)(rows, cols,  MATRIX_DICT_D)
        elif storage==MATRIX_DICT_TA:
            self.T = type(self)(cols, rows,  MATRIX_DICT_A)
            self.A = self
            self.M = type(self)(rows, cols,  MATRIX_DICT_T)
            self.D = type(self)(rows, cols,  MATRIX_DICT_TD)
        elif storage==MATRIX_DICT_D:
            self.T = type(self)(cols, rows,  MATRIX_DICT_T)
            self.A = type(self)(rows, cols,  MATRIX_DICT_A)
            self.M = type(self)(rows, cols,  MATRIX_DICT)
            self.D = self
        elif storage==MATRIX_DICT_TD:
            self.T = type(self)(cols, rows,  MATRIX_DICT)
            self.A = type(self)(rows, cols,  MATRIX_DICT_TA)
            self.M = type(self)(rows, cols,  MATRIX_DICT_T)
            self.D = self
        else:
            raise NotImplementedError(`storage`) #pragma NO COVER

def totree(obj, tab=''):
    from .core import Expr
    if isinstance(obj, Expr):
        head, data = obj.pair
        s = '%s%s head:%r\n' % (tab, type(obj).__name__, head)
        s += '%s' % (totree(data, tab+'  '))
        return s
    elif isinstance(obj, dict):
        return '%s%s:\n%s' % (tab, type(obj).__name__, '\n'.join([totree(item, tab+'  ') for item in obj.iteritems()]))
    elif isinstance(obj, (list, tuple)):
        return '%s%s:\n%s' % (tab, type(obj).__name__, '\n'.join([totree(item, tab+'  ') for item in obj]))
    else:
        return '%s%s:%r' % (tab, type(obj).__name__, obj)


def test_operations(operands, expected_results, unary_operations, binary_operations):
    from sympycore import Expr

    results = {}
    for line in expected_results.split('\n'):
        line = line.strip()
        if ':' not in line: continue
        expr, result = line.split(':')
        for e in expr.split(';'):
            e = e.strip()
            results[e] = [r.strip() for r in result.split(';')]

    if isinstance(operands, tuple):
        operands1, operands2 = operands
    else:
        operands1 = operands2 = operands

    for op1 in operands1:
        if isinstance(op1, Expr):
            for op in unary_operations:
                expr = '%s(%s)' % (op, op1) 

                try:
                    result_obj = eval('%s(op1)' % (op), dict(op1=op1))
                    result = str(result_obj)
                except Exception, msg:
                    print  expr,'failed with %s' % (msg)
                    raise
                
                if expr not in results:
                    print '%s:%s' % (expr, result)
                    continue
                try:
                    assert result in results[expr], `results[expr], result`
                except AssertionError:
                    print 
                    print 'op1: %s' % (op1)
                    print totree(op1, '  ')
                    print '%r result: %s' % (op, result_obj)
                    print totree(result_obj, '  ')
                    raise
        for op2 in operands2:
            if not (isinstance(op1, Expr) or isinstance(op2, Expr)):
                continue
            for op in binary_operations:
                expr = '(%s)%s(%s)' % (op1, op, op2)

                result_obj = None
                try:
                    result_obj = eval('(op1)%s(op2)' % op, dict(op1=op1, op2=op2))
                    result = str(result_obj)
                except Exception, msg:
                    result = str(msg)
                    if not result.startswith('unsupported'):
                        print  expr,`op1,op,op2`,'failed with %s' % (msg)
                        raise
                if expr not in results:
                    print '%s:%s' % (expr, result)
                    continue
                if result.startswith('unsupported'):
                    assert 'unsupported' in results[expr], `results[expr], result, op1, op2, expr`
                else:
                    try:
                        assert result in results[expr], `results[expr], result`
                    except AssertionError:
                        print
                        print 'op1: %s' % (op1)
                        print totree(op1, '  ')
                        print 'op2: %s' % (op2)
                        print totree(op2, '  ')
                        print '%r result: %s' % (op, result_obj)
                        print totree(result_obj, '  ')
                        raise


from heads import *



""" Provides calculus support.
"""
__docformat__ = "restructuredtext"

from .algebra import Calculus, I
from .infinity import  oo, undefined, moo, zoo
from .functions import (Exp, Log, Sqrt, Sin, Cos, Tan, Cot, pi, E, gamma,
    Sign, Mod, Ln, Factorial, ArcSin)
from .functions import CalculusFunctionRing, CalculusDifferentialRing, CalculusOperatorRing

def diff(expr, symbol, order=1):
    return expr.diff(symbol, order)

from .relational import Assumptions

Symbol = Calculus.Symbol

def Number(num, denom=None):
    n = Calculus.Number(Calculus.convert_coefficient(num))
    if denom is None:
        return n
    return n / denom

def Rational(num, denom):
    #XXX: use Div
    return Calculus.Number(Calculus.convert_coefficient(num)) / denom

Add = Calculus.Add
Mul = Calculus.Mul
Pow = Calculus.Pow

from ..arithmetic.number_theory import factorial as _factorial
def factorial(n):
    return Number(_factorial(n))
    
Polynom = Calculus.Polynom




__all__ = ['Ring', 'CommutativeRing']

from ..basealgebra import Algebra
from .interface import RingInterface

from ..core import init_module, classes
init_module.import_heads()
init_module.import_numbers()

@init_module
def _init(m):
    from ..arithmetic import mpq
    Ring.coefftypes = (int, long, mpq)

class Ring(Algebra, RingInterface):
    """
    Ring represents algebraic ring (R, +, *) where (R, +) is abelian
    group, (R,*) is monoid, with distributivity.
    """

    @classmethod
    def get_function_algebra(cls):
        return classes.FunctionRing

    def __str__(self):
        h, d = self.pair
        return h.data_to_str_and_precedence(type(self), d)[0]

    def __pos__(self):
        return self

    def __neg__(self):
        return self.head.neg(type(self), self)

    def __add__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.add_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.add(cls, self, other)

    __radd__ = __add__

    def __iadd__(self, other):
        cls = type(self)
        if type(other) is not cls:
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.inplace_add(cls, self, other)

    def __sub__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.sub_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.sub(cls, self, other)

    def __rsub__(self, other):
        return other + (-self)

    def __isub__(self, other):
        cls = type(self)
        if type(other) is not cls:
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.inplace_add(cls, self, -other)

    def __mul__(self, other):
        cls = type(self)
        tother = type(other)
        if cls is not tother:
            if tother in numbertypes_set:
                return self.head.non_commutative_mul_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.non_commutative_mul(cls, self, other)

    def __rmul__(self, other):
        cls = type(self)
        tother = type(other)
        if cls is not tother:
            if tother in numbertypes_set:
                return self.head.non_commutative_rmul_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return other.head.non_commutative_mul(cls, other, self)

    def __pow__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.pow_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.pow(cls, self, other)

    def __rpow__(self, other):
        cls = type(self)
        tother = type(other)
        if cls is not tother:
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return other.head.pow(cls, other, self)

    def __div__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.non_commutative_div_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self * other**-1

    def __rdiv__(self, other):
        cls = type(self)
        tother = type(other)
        if cls is not tother:
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return other * self**-1

    __truediv__ = __div__

    def expand(self):
        return self.head.expand(type(self), self)

    def evalf(self, n=None):
        return self.head.evalf(type(self), self, n)

class CommutativeRing(Ring):

    def __mul__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.commutative_mul_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented:
                return NotImplemented
        return self.head.commutative_mul(cls, self, other)

    __rmul__ = __mul__

    def __imul__(self, other):
        cls = type(self)
        if type(other) is not cls:
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented:
                return NotImplemented
        return self.head.inplace_commutative_mul(cls, self, other)

    def __div__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.commutative_div_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented:
                return NotImplemented
        return self.head.commutative_div(cls, self, other)

    def __rdiv__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.commutative_rdiv_number(cls, self, other)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented:
                return NotImplemented
        return other * self**-1

    def to(self, target, *args):
        """ Convert expression to target representation.

        The following targets are recognized:

          EXP_COEFF_DICT - convert expression to exponents-coefficient
            representation, additional arguments are variables. When
            no arguments are specified, variables will be all symbols
            and non-power expressions used in the expression.

          TERM_COEFF_DICT - convert expression to term-coefficient
            representation. Note that the returned result may have
            actual head NUMBER, SYMBOL, TERM_COEFF, POW, or
            BASE_EXP_DICT instead of TERM_COEFF_DICT.
        """
        head, data = self.pair
        if target is head:
            return self
        if target is EXP_COEFF_DICT:
            return head.to_EXP_COEFF_DICT(type(self), data, self, args or None)
        if target is TERM_COEFF_DICT:
            return head.to_TERM_COEFF_DICT(type(self), data, self)
        raise NotImplementedError('%s.to(target=%r)' % (type(self), target))

    def diff(self, symbol, order=1):
        if order==0:
            return self
        cls = type(self)
        if type(symbol) is cls:
            assert symbol.head is SYMBOL,`symbol.pair`
            symbol = symbol.data
        elif isinstance(symbol, str):
            pass
        else:
            raise TypeError('diff(symbol, order) first argument must be str or %s instance but got %s instance' % (cls.__name__, type(symbol).__name__))
        try:
            cache = {}
            result = self.head.diff(cls, self.data, self, symbol, order, cache=cache)
        finally:
            cache.clear()
        return result

    def integrate(self, x):
        cls = type(self)

        t = type(x)
        if t is tuple:
            x, a, b = x
            t = type(x)
        else:
            a, b = None, None

        if t is cls:
            assert x.head is SYMBOL,`x.pair`
            x = x.data
        elif t is str:
            pass
        else:
            raise TypeError('integrate(x,..), x must be str or %s instance but got %s instance' % (cls.__name__, type(symbol).__name__))
    
        if a is None:
            return self.head.integrate_indefinite(cls, self.data, self, x)
        if type(a) is not cls:
            a = cls(a)
        if type(b) is not cls:
            b = cls(b)
        return self.head.integrate_definite(cls, self.data, self, x, a, b)

classes.Ring = Ring
classes.CommutativeRing = CommutativeRing




__all__ = ['Group', 'AdditiveGroup', 'AdditiveAbelianGroup']

from ..basealgebra import Algebra
from ..core import init_module, classes
init_module.import_heads()
init_module.import_numbers()

class Group(Algebra):
    """
    Group represents algebraic group (G, <op>) where <op> is a binary
    operation that is associative, G contains identity element and
    have inverse elements.
    """
    algebra_options = dict(evaluate_addition = True,
                           evaluate_multiplication = True,
                           is_additive_group = False,
                           is_multiplicative_group = False,
                           )

    def to(self, target, *args):
        """ Convert expression to target representation.

        The following targets are recognized:

          EXP_COEFF_DICT - convert expression to exponents-coefficient
            representation, additional arguments are variables. When
            no arguments are specified, variables will be all symbols
            and non-power expressions used in the expression.

          TERM_COEFF_DICT - convert expression to term-coefficient
            representation. Note that the returned result may have
            actual head NUMBER, SYMBOL, TERM_COEFF, POW, or
            BASE_EXP_DICT instead of TERM_COEFF_DICT.
        """
        head, data = self.pair
        if target is EXP_COEFF_DICT:
            return head.to_EXP_COEFF_DICT(type(self), data, self, args or None)
        if target is TERM_COEFF_DICT:
            return head.to_TERM_COEFF_DICT(type(self), data, self)
        if target is ADD:
            return head.to_ADD(type(self), data, self)
        if target is MUL:
            return head.to_MUL(type(self), data, self)
        raise NotImplementedError('%s.to(target=%r)' % (type(self), target))

class AdditiveGroup(Group):
    """
    AdditiveGroup is a group with addition ``+`` operation, 0 is
    identity element and ``-`` denotes inverse. Multiplication ``*``
    is defined only with numbers (integers) that are considered as
    coefficients.

    Warning: AdditiveGroup allows operating with numbers although
    mathematically strictly speaking they are not defined, in general.
    E.g. the only number that belongs to additive group, is 0. Hence
    ``1 + 1`` does not make sense unless additive group represents
    specifically the set of integers.

    AdditiveGroup expressions have default heads NUMBER, SYMBOL, NEG,
    ADD, TERM_COEFF. When evaluation is disabled (see
    UnevaluatedAddition context) then MUL head will be used as well.
    """
    algebra_options = Group.algebra_options.copy()
    algebra_options.update(is_additive_group_commutative = False,
                           is_additive_group = True)

    def __pos__(self):
        return self.head.algebra_pos(type(self), self)

    def __neg__(self):
        return self.head.algebra_neg(type(self), self)

    def __add__(self, other, inplace=False):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.algebra_add_number(cls, self, other, inplace)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.algebra_add(cls, self, other, inplace)

    def __radd__(self, other):
        cls = type(self)
        other = cls.convert(other, typeerror=False)
        if other is NotImplemented: return NotImplemented
        return other.head.algebra_add(cls, other, self, False)

    def __iadd__(self, other):
        return self.__add__(other, self.is_writable)

    def __sub__(self, other, inplace=False):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.algebra_add_number(cls, self, -other, inplace)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.algebra_add(cls, self, -other, inplace)

    def __rsub__(self, other):
        return other + (-self)

    def __isub__(self, other):
        return self.__add__(-other, self.is_writable)

    def __mul__(self, other, inplace=False):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.algebra_mul_number(cls, self, other, inplace)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented:
                return NotImplemented
        if self.head is NUMBER or other.head is NUMBER:
            return self.head.algebra_mul(cls, self, other, inplace)
        raise TypeError('unsupported operand type(s) for *: %r and %r' \
                        % (self.pair, other.pair))

    def __rmul__(self, other):
        cls = type(self)
        other = cls.convert(other, typeerror=False)
        if other is NotImplemented:
            return NotImplemented
        if self.head is NUMBER or other.head is NUMBER:
            return other.head.algebra_mul(cls, other, self, False)
        raise TypeError('unsupported operand type(s) for *: %r and %r' \
                        % (type(other).__name__, cls.__name__))

    def __imul__(self, other):
        return self.__mul__(other, self.is_writable)

class AdditiveAbelianGroup(AdditiveGroup):
    """
    AdditiveAbelianGroup is an additive group with commutative
    addition.

    AdditiveAbelianGroup expressions have default heads NUMBER,
    SYMBOL, TERM_COEFF, TERM_COEFF_DICT.
    """

    algebra_options = AdditiveGroup.algebra_options.copy()
    algebra_options.update(is_additive_group_commutative = True)


class MultiplicativeGroup(Group):
    """
    MultiplicativeGroup is a Group with * operation, 1 is identity element
    and ``**(-1)`` denotes inverse. 
    """
    algebra_options = Group.algebra_options.copy()
    algebra_options.update(is_multiplicative_group_commutative = False,
                           is_multiplicative_group = True)

    def __mul__(self, other, inplace=False):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.algebra_mul_number(cls, self, other, inplace)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.algebra_mul(cls, self, other, inplace)

    def __rmul__(self, other):
        cls = type(self)
        other = cls.convert(other, typeerror=False)
        if other is NotImplemented: return NotImplemented
        return other.head.algebra_mul(cls, other, self, False)

    def __div__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                pass
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self * other ** (-1)

    def __rdiv__(self, other):
        cls = type(self)
        other = cls.convert(other, typeerror=False)
        if other is NotImplemented: return NotImplemented
        return other * self ** (-1)

    def __pow__(self, other, inplace=False):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.algebra_pow_number(cls, self, other, inplace)
            other = cls.convert(other, typeerror=False)
            if other is NotImplemented: return NotImplemented
        return self.head.algebra_pow(cls, self, other, inplace)

classes.Group = Group
classes.AdditiveGroup = AdditiveGroup
classes.AdditiveAbelianGroup = AdditiveAbelianGroup
classes.MultiplicativeGroup = MultiplicativeGroup





__all__ = ['OperatorRing']

from ..core import classes, init_module
from .algebra import FunctionRing

init_module.import_heads()
init_module.import_numbers()

class OperatorRing(FunctionRing):

    @classmethod
    def convert(cls, obj, typeerror=True):
        if isinstance(obj, cls):
            return obj
        fcls = cls.get_value_algebra()
        if isinstance(obj, fcls):
            return cls(NUMBER, obj)
        if isinstance(obj, fcls.get_value_algebra()):
            return cls(NUMBER, obj)
        if isinstance(obj, (str, unicode, classes.Verbatim)):
            return classes.Verbatim.convert(obj).as_algebra(cls)
        return cls.handle_convert_failure('algebra', obj, typeerror)

    def as_algebra(self, cls, typeerror=True):
        raise NotImplementedError(`cls`)

    @classmethod
    def get_value_algebra(cls):
        return classes.FunctionRing

    @classmethod
    def get_function_algebra(cls):
        raise NotImplementedError(`cls`)

    @classmethod
    def get_differential_algebra(cls):
        raise NotImplementedError(`cls`)

classes.OperatorRing = OperatorRing



"""Provides low-level evalf function.
"""

__all__ = ['evalf']

import math
import cmath
import re
import mpmath

int_pattern = re.compile('\d+([.]\d+)?')

def quote_numbers(s, format):
    return int_pattern.sub(lambda m: (format % m.group()), s)

def replace_names(s, names):
    for name, new in names.items():
        s = s.replace(name, new)
    return s

def f_header(symbols):
    if isinstance(symbols, (tuple, list)):
        return "lambda %s: " % ",".join(map(str, symbols))
    if not symbols:
        return "lambda: "
    return "lambda %s: " % symbols

def convert_mpmath(expr):
    s = str(expr)
    s = quote_numbers(s, "mpf(%s)")
    s = replace_names(s, { 'I':'j', 'E':'e', 'oo':'inf','undefined':'nan',
                           'Sin':'sin','Cos':'cos', 'Exp':'exp'})
    return s

def compile_mpmath(symbols, expr):
    s = convert_mpmath(expr)
    f = eval(f_header(symbols) + s, vars(mpmath))
    return f

def evalf(expr, digits=15):
    s = convert_mpmath(expr)
    return eval(s, vars(mpmath))




from sympycore import Ring, core

def test_add():
    x,y,z = map(Ring, 'xyz')
    assert Ring.Add(x,y) == x+y

def test_div():
    x,y,z = map(Ring, 'xyz')
    assert Ring.Div(x,y) == x/y, str(Ring.Div(x,y))

def test_polynom():
    x,y,z = map(Ring, 'xyz')
    assert Ring.Polynom(x).data==(('x',), {core.IntegerList([1]):1})
    assert Ring.Polynom(x, variables=(x,y)).data==(('x','y'), {core.IntegerList([1,0]):1})



"""
Plotting (requires matplotlib)
"""

from colorsys import hsv_to_rgb, hls_to_rgb
from .libmp import NoConvergence
from .libmp.backend import xrange

class VisualizationMethods(object):
    plot_ignore = (ValueError, ArithmeticError, ZeroDivisionError, NoConvergence)

def plot(ctx, f, xlim=[-5,5], ylim=None, points=200, file=None, dpi=None,
    singularities=[], axes=None):
    r"""
    Shows a simple 2D plot of a function `f(x)` or list of functions
    `[f_0(x), f_1(x), \ldots, f_n(x)]` over a given interval
    specified by *xlim*. Some examples::

        plot(lambda x: exp(x)*li(x), [1, 4])
        plot([cos, sin], [-4, 4])
        plot([fresnels, fresnelc], [-4, 4])
        plot([sqrt, cbrt], [-4, 4])
        plot(lambda t: zeta(0.5+t*j), [-20, 20])
        plot([floor, ceil, abs, sign], [-5, 5])

    Points where the function raises a numerical exception or
    returns an infinite value are removed from the graph.
    Singularities can also be excluded explicitly
    as follows (useful for removing erroneous vertical lines)::

        plot(cot, ylim=[-5, 5])   # bad
        plot(cot, ylim=[-5, 5], singularities=[-pi, 0, pi])  # good

    For parts where the function assumes complex values, the
    real part is plotted with dashes and the imaginary part
    is plotted with dots.

    .. note :: This function requires matplotlib (pylab).
    """
    if file:
        axes = None
    fig = None
    if not axes:
        import pylab
        fig = pylab.figure()
        axes = fig.add_subplot(111)
    if not isinstance(f, (tuple, list)):
        f = [f]
    a, b = xlim
    colors = ['b', 'r', 'g', 'm', 'k']
    for n, func in enumerate(f):
        x = ctx.arange(a, b, (b-a)/float(points))
        segments = []
        segment = []
        in_complex = False
        for i in xrange(len(x)):
            try:
                if i != 0:
                    for sing in singularities:
                        if x[i-1] <= sing and x[i] >= sing:
                            raise ValueError
                v = func(x[i])
                if ctx.isnan(v) or abs(v) > 1e300:
                    raise ValueError
                if hasattr(v, "imag") and v.imag:
                    re = float(v.real)
                    im = float(v.imag)
                    if not in_complex:
                        in_complex = True
                        segments.append(segment)
                        segment = []
                    segment.append((float(x[i]), re, im))
                else:
                    if in_complex:
                        in_complex = False
                        segments.append(segment)
                        segment = []
                    if hasattr(v, "real"):
                        v = v.real
                    segment.append((float(x[i]), v))
            except ctx.plot_ignore:
                if segment:
                    segments.append(segment)
                segment = []
        if segment:
            segments.append(segment)
        for segment in segments:
            x = [s[0] for s in segment]
            y = [s[1] for s in segment]
            if not x:
                continue
            c = colors[n % len(colors)]
            if len(segment[0]) == 3:
                z = [s[2] for s in segment]
                axes.plot(x, y, '--'+c, linewidth=3)
                axes.plot(x, z, ':'+c, linewidth=3)
            else:
                axes.plot(x, y, c, linewidth=3)
    axes.set_xlim([float(_) for _ in xlim])
    if ylim:
        axes.set_ylim([float(_) for _ in ylim])
    axes.set_xlabel('x')
    axes.set_ylabel('f(x)')
    axes.grid(True)
    if fig:
        if file:
            pylab.savefig(file, dpi=dpi)
        else:
            pylab.show()

def default_color_function(ctx, z):
    if ctx.isinf(z):
        return (1.0, 1.0, 1.0)
    if ctx.isnan(z):
        return (0.5, 0.5, 0.5)
    pi = 3.1415926535898
    a = (float(ctx.arg(z)) + ctx.pi) / (2*ctx.pi)
    a = (a + 0.5) % 1.0
    b = 1.0 - float(1/(1.0+abs(z)**0.3))
    return hls_to_rgb(a, b, 0.8)

def cplot(ctx, f, re=[-5,5], im=[-5,5], points=2000, color=None,
    verbose=False, file=None, dpi=None, axes=None):
    """
    Plots the given complex-valued function *f* over a rectangular part
    of the complex plane specified by the pairs of intervals *re* and *im*.
    For example::

        cplot(lambda z: z, [-2, 2], [-10, 10])
        cplot(exp)
        cplot(zeta, [0, 1], [0, 50])

    By default, the complex argument (phase) is shown as color (hue) and
    the magnitude is show as brightness. You can also supply a
    custom color function (*color*). This function should take a
    complex number as input and return an RGB 3-tuple containing
    floats in the range 0.0-1.0.

    To obtain a sharp image, the number of points may need to be
    increased to 100,000 or thereabout. Since evaluating the
    function that many times is likely to be slow, the 'verbose'
    option is useful to display progress.

    .. note :: This function requires matplotlib (pylab).
    """
    if color is None:
        color = ctx.default_color_function
    import pylab
    if file:
        axes = None
    fig = None
    if not axes:
        fig = pylab.figure()
        axes = fig.add_subplot(111)
    rea, reb = re
    ima, imb = im
    dre = reb - rea
    dim = imb - ima
    M = int(ctx.sqrt(points*dre/dim)+1)
    N = int(ctx.sqrt(points*dim/dre)+1)
    x = pylab.linspace(rea, reb, M)
    y = pylab.linspace(ima, imb, N)
    # Note: we have to be careful to get the right rotation.
    # Test with these plots:
    #   cplot(lambda z: z if z.real < 0 else 0)
    #   cplot(lambda z: z if z.imag < 0 else 0)
    w = pylab.zeros((N, M, 3))
    for n in xrange(N):
        for m in xrange(M):
            z = ctx.mpc(x[m], y[n])
            try:
                v = color(f(z))
            except ctx.plot_ignore:
                v = (0.5, 0.5, 0.5)
            w[n,m] = v
        if verbose:
            print(n, "of", N)
    rea, reb, ima, imb = [float(_) for _ in [rea, reb, ima, imb]]
    axes.imshow(w, extent=(rea, reb, ima, imb), origin='lower')
    axes.set_xlabel('Re(z)')
    axes.set_ylabel('Im(z)')
    if fig:
        if file:
            pylab.savefig(file, dpi=dpi)
        else:
            pylab.show()

def splot(ctx, f, u=[-5,5], v=[-5,5], points=100, keep_aspect=True, \
          wireframe=False, file=None, dpi=None, axes=None):
    """
    Plots the surface defined by `f`.

    If `f` returns a single component, then this plots the surface
    defined by `z = f(x,y)` over the rectangular domain with
    `x = u` and `y = v`.

    If `f` returns three components, then this plots the parametric
    surface `x, y, z = f(u,v)` over the pairs of intervals `u` and `v`.

    For example, to plot a simple function::

        >>> from mpmath import *
        >>> f = lambda x, y: sin(x+y)*cos(y)
        >>> splot(f, [-pi,pi], [-pi,pi])    # doctest: +SKIP

    Plotting a donut::

        >>> r, R = 1, 2.5
        >>> f = lambda u, v: [r*cos(u), (R+r*sin(u))*cos(v), (R+r*sin(u))*sin(v)]
        >>> splot(f, [0, 2*pi], [0, 2*pi])    # doctest: +SKIP

    .. note :: This function requires matplotlib (pylab) 0.98.5.3 or higher.
    """
    import pylab
    import mpl_toolkits.mplot3d as mplot3d
    if file:
        axes = None
    fig = None
    if not axes:
        fig = pylab.figure()
        axes = mplot3d.axes3d.Axes3D(fig)
    ua, ub = u
    va, vb = v
    du = ub - ua
    dv = vb - va
    if not isinstance(points, (list, tuple)):
        points = [points, points]
    M, N = points
    u = pylab.linspace(ua, ub, M)
    v = pylab.linspace(va, vb, N)
    x, y, z = [pylab.zeros((M, N)) for i in xrange(3)]
    xab, yab, zab = [[0, 0] for i in xrange(3)]
    for n in xrange(N):
        for m in xrange(M):
            fdata = f(ctx.convert(u[m]), ctx.convert(v[n]))
            try:
                x[m,n], y[m,n], z[m,n] = fdata
            except TypeError:
                x[m,n], y[m,n], z[m,n] = u[m], v[n], fdata
            for c, cab in [(x[m,n], xab), (y[m,n], yab), (z[m,n], zab)]:
                if c < cab[0]:
                    cab[0] = c
                if c > cab[1]:
                    cab[1] = c
    if wireframe:
        axes.plot_wireframe(x, y, z, rstride=4, cstride=4)
    else:
        axes.plot_surface(x, y, z, rstride=4, cstride=4)
    axes.set_xlabel('x')
    axes.set_ylabel('y')
    axes.set_zlabel('z')
    if keep_aspect:
        dx, dy, dz = [cab[1] - cab[0] for cab in [xab, yab, zab]]
        maxd = max(dx, dy, dz)
        if dx < maxd:
            delta = maxd - dx
            axes.set_xlim3d(xab[0] - delta / 2.0, xab[1] + delta / 2.0)
        if dy < maxd:
            delta = maxd - dy
            axes.set_ylim3d(yab[0] - delta / 2.0, yab[1] + delta / 2.0)
        if dz < maxd:
            delta = maxd - dz
            axes.set_zlim3d(zab[0] - delta / 2.0, zab[1] + delta / 2.0)
    if fig:
        if file:
            pylab.savefig(file, dpi=dpi)
        else:
            pylab.show()


VisualizationMethods.plot = plot
VisualizationMethods.default_color_function = default_color_function
VisualizationMethods.cplot = cplot
VisualizationMethods.splot = splot



"""
Writes a .mod file template for the user to fill in.

Notes
-----
This requires Python 2.5 or higher.
"""
import os
from string import Template

str_template = Template("""%Model Information++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
$helpinfo
%Parameters+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
$helpparam
%Variable Vectors+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
$helpvar
%Boundary Conditions++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
$helpbound
%Variable Substitution Non-Linear System++++++++++++++++++++++++++++++++++++++++
$helpsub
%Non-Linear First-Order Conditions++++++++++++++++++++++++++++++++++++++++++++++
$helpfoc
%Steady State Non-Linear System [Manual]++++++++++++++++++++++++++++++++++++++++
$helpssnl
%Steady States [Closed Form]++++++++++++++++++++++++++++++++++++++++++++++++++++
$helpsscl
%Log-Linearized Model Equations+++++++++++++++++++++++++++++++++++++++++++++++++
$helploglin
%Variance-Covariance Matrix+++++++++++++++++++++++++++++++++++++++++++++++++++++
$helpvarcov
%Minford Model Evaluation+++++++++++++++++++++++++++++++++++++++++++++++++++++++
$helpminf
%End of Model File++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
""")

nohelp=dict(zip(['helpinfo', 'helpparam', 'helpvar', 'helpbound', 
                'helpsub', 'helpfoc', 'helpssnl', 'helpsscl', 'helploglin', 
                'helpvarcov', 'helpminf'], ("",)*13))

helpinfo = """Available information: (Description, Name)
Example:
Description = This is the canonical Christiano, Eichenbaum and Evans New-Keynesian model;
Name = CEE NK Model;
"""
helpparam = """Example: 
rho = .88;
alpha_phi = .45;

Do NOT use lambda as a parameter name.
For now, make sure that you do not do integer division in the parameters. 
I.e., 1/4 will give 0 not .25.  If you want .25, do 1./4 or 1/4.
"""

def _overwrite(fname):
    overwrite = raw_input("File %s exists. Overwrite? (y/n) > " % fname) 
    overwrite = overwrite.lower()
    if overwrite != 'y' and overwrite != 'n':
        print "Please type 'y' or 'n'"
        _overwrite(fname)
    if 'y' in overwrite:
        return True
    if 'n' in overwrite:
        return False

def _get_new_name(fname):
    """
    Checks if fname exists and decide to overwrite.
    """
    overwrite = _overwrite(fname)
    if overwrite:
        return fname
    else:
        newfname = raw_input("Select a new filename (Do not respecify the path) > ")
        if not newfname.endswith('.mod'):
            newfname += '.mod'
        path = os.path.dirname(fname)
        newfname = os.path.join(path, newfname)
        if os.path.exists(newfname):
            newfname = _get_new_name(newfname)
        return newfname

def make_modfile(fname, help=False):
    """
    Creates a template .mod file for the user to edit.

    Parameters
    ----------
    fname : str
        Path and filename for new .mod file.  ".mod" extension is optional.
    help : bool, optional
        Whether or not to include some guidance within the template on how to
        edit the .mod file.
    """
    if not fname.endswith('.mod'):
        fname += ".mod"
    if os.path.exists(fname):
        fname = _get_new_name(fname)
    with open(fname, "w") as modfile:
        if not help:
            modstr = str_template.substitute(nohelp)
        else:
            raise ValueError("Help isn't written yet")
        modfile.writelines(modstr)




__all__ = ['ADD']

from .base import heads_precedence, ArithmeticHead

from ..core import init_module
init_module.import_heads()
init_module.import_numbers()
init_module.import_lowlevel_operations()

@init_module
def _init(module):
    from ..arithmetic.number_theory import multinomial_coefficients
    module.multinomial_coefficients = multinomial_coefficients

class AddHead(ArithmeticHead):

    """
    AddHead represents addition n-ary operation where operands is
    given as a n-sequence of expressions. For example, expression 'a +
    2*b' is 'Expr(ADD, (a, 2*b))' where ADD=AddHead()
    """

    op_mth = '__add__'
    op_rmth = '__radd__'

    def is_data_ok(self, cls, data):
        if type(data) in [tuple, list]:
            for a in data:
                if not isinstance(a, cls):
                    return '%s data item must be %s instance but got %s' % (self, cls, type(a))
        else:
            return '%s data part must be a list but got %s' % (self, type(data))
    
    def __repr__(self): return 'ADD'

    def new(self, cls, operands, evaluate=True):
        if not evaluate:
            n = len(operands)
            if n==1:
                return operands[0]
            if n==0:
                return cls(NUMBER, 0)
            return cls(self, operands)
        d = {}
        l = []
        operands = list(operands)
        while operands:
            op = operands.pop(0)
            if op==0:
                continue
            head, data = op.pair
            if head is ADD:
                operands.extend(data)
                continue
            elif head is SUB:
                operands.append(data[0])
                for o in data[1:]:
                    operands.append(-o)
                continue
            elif head is TERM_COEFF_DICT:
                for term, coeff in data.iteritems():
                    n = len(d)
                    dict_add_item(cls, d, term, coeff)
                    if n < len(d):
                        l.append(term)
                    elif n > len(d):
                        l.remove(term)
            else:
                term, coeff = op.head.term_coeff(cls, op)
                n = len(d)
                dict_add_item(cls, d, term, coeff)
                if n < len(d):
                    l.append(term)
                elif n > len(d):
                    l.remove(term)
        r = []
        one = cls(NUMBER, 1)
        for term in l:
            r.append(TERM_COEFF.new(cls, (term, d[term])))
        m = len(r)
        if m==0:
            return cls(NUMBER, 0)
        if m==1:
            return r[0]
        return cls(self, r)

    def reevaluate(self, cls, operands):
        r = cls(NUMBER, 0)
        for op in operands:
            r += op
        return r

    def data_to_str_and_precedence(self, cls, operands):
        m = len(operands)
        if m==0:
            return '0', heads_precedence.NUMBER
        if m==1:
            op = operands[0]
            return op.head.data_to_str_and_precedence(cls, op.data)
        add_p = heads_precedence.ADD
        r = ''
        evaluate_addition = cls.algebra_options.get('evaluate_addition')
        for op in operands:
            t,t_p = op.head.data_to_str_and_precedence(cls, op.data)
            if not r:
                r += '(' + t + ')' if t_p < add_p else t
            elif evaluate_addition and t.startswith('-'):
                r += ' - ' + t[1:]
            else:
                r += ' + (' + t + ')' if t_p < add_p else ' + ' + t
        return r, add_p

    def term_coeff(self, cls, expr):
        term_list = expr.data
        if not term_list:
            return cls(NUMBER, 0), 1
        if len(term_list)==1:
            expr = term_list[0]
            return expr.head.term_coeff(cls, expr)
        return expr, 1

    def neg(self, cls, expr):
        return cls(ADD, [-term for term in expr.data])

    def add(self, cls, lhs, rhs):
        term_list = lhs.data
        rhead, rdata = rhs.pair
        if rhead is ADD:
            return ADD.new(cls, lhs.data + rdata)
        if rhead is TERM_COEFF_DICT:
            rdata = [t * c for t, c in rdata.items()]
            return ADD.new(cls, lhs.data + rdata)
        if rhead is SUB:
            rdata = rdata[:1] + [-op for op in rdata[1:]]
            return ADD.new(cls, lhs.data + rdata)
        return ADD.new(cls, lhs.data + [rhs])

    inplace_add = add

    def sub(self, cls, lhs, rhs):
        return lhs + (-rhs)

    def commutative_mul(self, cls, lhs, rhs):
        rhead, rdata = rhs.pair
        if rhead is NUMBER:
            return ADD.new(cls, [op*rhs for op in lhs.data])
        if rhead is SYMBOL:
            return cls(BASE_EXP_DICT, {lhs:1, rhs:1})
        if rhead is ADD:
            if lhs==rhs:
                return lhs ** 2
            return cls(BASE_EXP_DICT, {lhs:1, rhs:1})
        if rhead is TERM_COEFF:
            term, coeff = rdata
            return (lhs * term) * coeff
        if rhead is POW:
            base, exp = rhs.data
            if lhs==base:
                return POW.new(cls, (base, exp + 1))
            return cls(BASE_EXP_DICT, {lhs:1, base:exp})
        if rhead is BASE_EXP_DICT:
            data = rdata.copy()
            dict_add_item(cls, data, lhs, 1)
            return BASE_EXP_DICT.new(cls, data)

        raise NotImplementedError(`self, lhs.pair, rhs.pair`)

    def commutative_mul_number(self, cls, lhs, rhs):
        return ADD.new(cls, [op*rhs for op in lhs.data])

    def pow(self, cls, base, exp):
        return POW.new(cls, (base, exp))

    pow_number = pow

    def walk(self, func, cls, data, target):
        l = []
        flag = False
        for op in data:
            o = op.head.walk(func, cls, op.data, op)
            if op is not o:
                flag = True
            l.append(o)
        if flag:
            r = ADD.new(cls, l)
            return func(cls, r.head, r.data, r)
        return func(cls, self, data, target)

    def scan(self, proc, cls, operands, target):
        for operand in operands:
            operand.head.scan(proc, cls, operand.data, target)
        proc(cls, self, operands, target)

    def expand(self, cls, expr):
        l = []
        for op in expr.data:
            h, d = op.pair
            l.append(h.expand(cls, op))
        return self.new(cls, l)

    def expand_intpow(self, cls, expr, intexp):
        if intexp<=1:
            return POW.new(cls, (expr, intexp))
        operands = expr.data
        mdata = multinomial_coefficients(len(operands), intexp)
        s = cls(NUMBER, 0)
        for exps, n in mdata.iteritems():
            m = cls(NUMBER, n)
            for i,e in enumerate(exps):
                m *= operands[i] ** e
            s += m
        return s

    def to_TERM_COEFF_DICT(self, Algebra, data, expr):
        s = Algebra(NUMBER, 0)
        for op in data:
            s += op.head.to_TERM_COEFF_DICT(Algebra, op.data, op)
        return s

    def to_ADD(self, Algebra, data, expr):
        return expr

    def algebra_pos(self, Algebra, expr):
        if Algebra.algebra_options.get('evaluate_addition'):
            if Algebra.algebra_options.get('is_additive_group_commutative'):
                return +ADD.to_TERM_COEFF_DICT(Algebra, expr.data, expr)
        return expr

    def algebra_neg(self, Algebra, expr):
        if Algebra.algebra_options.get('evaluate_addition'):
            if Algebra.algebra_options.get('is_additive_group_commutative'):
                return -ADD.to_TERM_COEFF_DICT(Algebra, expr.data, expr)
            return add_new(Algebra, [-op for op in expr.data[::-1]])
        return Algebra(NEG, expr)

    def combine_add_list(self, Algebra, data):
        """
        Combine add operands of an additive group in data.
        data will be changed in place.
        """
        commutative = Algebra.algebra_options.get('is_additive_group_commutative')
        if commutative:
            d = {}
            for op in data:
                term, coeff = op.head.term_coeff(Algebra, op)
                term_coeff_dict_add_item(Algebra, d, term, coeff)
            data[:] = [term_coeff_new(Algebra, term_coeff) for term_coeff in d.iteritems()]
        else:
            n = len(data)
            i0 = 0
            while 1:
                i = i0
                if i+1 >= n:
                    break
                lhs = data[i]
                rhs = data[i+1]
                lterm, lcoeff = lhs.head.term_coeff(Algebra, lhs)
                rterm, rcoeff = rhs.head.term_coeff(Algebra, rhs)
                if lterm==rterm:
                    coeff = lcoeff + rcoeff
                    if coeff:
                        del data[i+1]
                        data[i] = term_coeff_new(Algebra, (lterm, coeff))
                        i0 = i
                        n -= 1
                    else:
                        del data[i:i+2]
                        i0 = max(i - 1, 0)
                        n -= 2
                elif not rcoeff:
                    del data[i+1]
                    n -= 1
                    i0 = i
                elif not lcoeff:
                    del data[i]
                    n -= 1
                    i0 = max(i-1,0)
                else:
                    i0 += 1
        return data

    def algebra_add_number(self, Algebra, lhs, rhs, inplace):
        return self.algebra_add(Algebra, lhs, Algebra(NUMBER, rhs), inplace)
        
    def algebra_add(self, Algebra, lhs, rhs, inplace):
        rhead, rdata = rhs.pair
        if rhead is TERM_COEFF_DICT or rhead is EXP_COEFF_DICT or rhead is MUL or rhead is NEG:
            rhs = rhs.to(ADD)
            rhead, rdata = rhs.pair
        if inplace:
            data = lhs.data
        else:
            data = lhs.data[:]
        if rhead is ADD:
            data.extend(rdata)
        else:
            data.append(rhs)
        if Algebra.algebra_options.get('evaluate_addition'):
            self.combine_add_list(Algebra, data)
        if inplace:
            return add(Algebra, lhs)
        return add_new(Algebra, data)

    def algebra_mul_number(self, Algebra, lhs, rhs, inplace):
        ntype = type(rhs)
        if Algebra.algebra_options.get('is_additive_group_commutative'):
            if not rhs:
                return Algebra(NUMBER, 0)
            return ADD.to_TERM_COEFF_DICT(Algebra, lhs.data, lhs) * rhs
        else:
            if Algebra.algebra_options.get('evaluate_addition'):
                if rhs == 0:
                    return Algebra(NUMBER, 0)
                if rhs == 1:
                    return lhs
                if ntype in inttypes_set:
                    if rhs > 0:
                        # (x+y)*3 = x+y+x+y+x+y
                        # TODO: optimize (x+y+x)*3 = x+y+x+x+y++x+x+y+x = x+y+2*x+y+2*x+y+x
                        data = lhs.data * rhs
                    else:
                        data = [-op for op in (lhs.data * (-rhs))[::-1]]
                    self.combine_add_list(Algebra, data)
                    return add_new(Algebra, data)
                    
            return mul_new(Algebra, [lhs, Algebra(NUMBER, rhs)])

    def algebra_mul(self, Algebra, lhs, rhs, inplace):
        ldata = lhs.data
        if Algebra.algebra_options.get('is_additive_group_commutative'):
            return ADD.to_TERM_COEFF_DICT(Algebra, lhs.data, lhs) * rhs
        else:
            if Algebra.algebra_options.get('evaluate_addition'):
                rhead, rdata = rhs.pair
                if rhead is NUMBER:
                    return ADD.algebra_mul_number(Algebra, lhs, rdata, inplace)
                return super(type(self), self).algebra_mul(Algebra, lhs, rhs, inplace)
            return mul_new(Algebra, [lhs, rhs])
    
ADD = AddHead()



import helpers as HLP

# ERROR CLASSES
class Errbase(Exception):
	pass

class Uhlerr(Errbase):
	pass

class Dataerr(Errbase):
	pass

class TS_err(Errbase):
	def __init__(self,inmes):
		self.inmes = inmes
		now = HLP.now_is()
		if self.inmes == 'Singular Matrix':
			self.outmes = 'Singular Matrix XX!! Cannot solve for beta'
			print str(now) + '   ' + self.outmes
			
# ERROR CLASSES OUT


'''
.. module:: stats
   :platform: Linux
   :synopsis: Wrapper package module containing the VAR and FAVAR modules each of which contain their respective classes from which
              empirical macro models can be instantiated with data.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''



__all__ = ['DifferentialRing']

from ..core import classes, Expr, init_module, DefinedFunction
from .operator import OperatorRing

init_module.import_heads()
init_module.import_numbers()

class Diff(DefinedFunction):

    def __new__(cls, expr, symbol):
        return OperatorRing()

class DifferentialRing(OperatorRing):

    """
    DifferentialRing(DIFF, x) - partial derivative with respect to x
    DifferentialRing(FDIFF, n) - partial derivative with respect to n-th argument
    
    """

    @classmethod
    def convert(cls, obj, typeerror=True):
        if isinstance(obj, cls):
            return obj
        fcls = cls.get_value_algebra()
        if isinstance(obj, fcls):
            return cls(NUMBER, obj)
        if isinstance(obj, fcls.get_value_algebra()):
            return cls(NUMBER, obj)
        return cls.handle_convert_failure('algebra', obj, typeerror)

    def get_differentiation_symbol(self):
        def get_differential_data(cls, head, data, target):
            if head is DIFF or head is FDIFF:
                return data
            return target
        cls = type(self)
        s = self.head.walk(get_differential_data, cls, self.data, self)
        #print s, type(s)
        if 0 and type(s) is cls:
            raise TypeError('Expression does not contain differential instances')        
        return s

    def __mul__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.commutative_mul_number(cls, self, other)
            if isinstance(other, type(self.get_differentiation_symbol())):
                return self(other)
            return NotImplemented
        return self.head.commutative_mul(cls, self, other)

    def __rmul__(self, other):
        cls = type(self)
        tother = type(other)
        if tother is not cls:
            if tother in numbertypes_set:
                return self.head.commutative_rmul_number(cls, self, other)
            if isinstance(other, Expr):
                scls = type(d)
                if type(self.get_differentiation_symbol()) is tother:
                    return self.head.non_commutative_rmul_number(cls, self, other)
            return NotImplemented
        return self.head.commutative_mul(cls, self, other)

    def __call__(self, *args):
        cls = type(self)
        FAlgebra = self.get_value_algebra()
        if len(args)==1:
            expr = args[0]
            if not isinstance(expr, Expr):
                expr = FAlgebra(CALLABLE, expr)
            result = self.head.diff_apply(cls, self.data, self, expr)
            if result is not NotImplemented:
                return result
            return FAlgebra(APPLY, (self, (expr,)))
        new_args = []
        args_changed = False
        for a in args:
            if not isinstance(expr, Expr):
                a = FAlgebra(CALLABLE, a)
                args_changed = True
            new_args.append(a)
        if args_changed:
            args = tuple(new_args)
        return FAlgebra(APPLY, (self, args))

classes.DifferentialRing = DifferentialRing


class FDFactory(object):

    def __new__(cls, DAlgebra=DifferentialRing, _cache={}):
        obj = _cache.get(DAlgebra)
        if obj is None:
            obj = object.__new__(cls)
            obj.DAlgebra = DAlgebra
            _cache[DAlgebra] = obj
        return obj
    def __str__(self): return 'FD'
    def __repr__(self): return 'FDFactory(%s)' % (self.DAlgebra.__name__)
    def __getitem__(self, index):
        return self.DAlgebra(FDIFF, self.DAlgebra.get_value_algebra()(index))

class DFactory(object):

    def __new__(cls, DAlgebra=DifferentialRing, _cache={}):
        obj = _cache.get(DAlgebra)
        if obj is None:
            obj = object.__new__(cls)
            obj.DAlgebra = DAlgebra
            _cache[DAlgebra] = obj
        return obj
    def __str__(self): return 'D'
    def __repr__(self): return 'DFactory(%s)' % (self.DAlgebra.__name__)
    def __getitem__(self, symbol):
        return self.DAlgebra(DIFF, self.DAlgebra.get_value_algebra()(symbol))




'''
.. module:: filters
   :platform: Linux
   :synopsis: Wrapper package module for all of the time series filters we make available in extracting the cycle and trend from
              (mostly) simulated data. But these filters are also imported into the macroeconometric class, like VAR and FAVAR to deal
              with real data.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''

from .hpfilter import hpfilter
from .bkfilter import bkfilter
from .cffilter import cffilter


from pymaclab.filters._hpfilter import hpfilter as hpf
import numpy as np

def hpfilter(data=None,lam=1600):
    if type(data) != type(np.array([1,2,3])):
        data = np.array(data)
    tlen = len(data)
    wdata = np.zeros((tlen,3))
    lam = 1600    
    return hpf(data,wdata,tlen,lam,0)


# The py library is part of the "py.test" testing suite (python-codespeak-lib
# on Debian), see http://codespeak.net/py/

import py

#this makes py.test put mpath directory into the sys.path, so that we can
#"import mpmath" from tests nicely
rootdir = py.magic.autopath().dirpath()



from ..libmp.backend import xrange
from .calculus import defun

#----------------------------------------------------------------------------#
#                                Polynomials                                 #
#----------------------------------------------------------------------------#

# XXX: extra precision
@defun
def polyval(ctx, coeffs, x, derivative=False):
    r"""
    Given coefficients `[c_n, \ldots, c_2, c_1, c_0]` and a number `x`,
    :func:`~mpmath.polyval` evaluates the polynomial

    .. math ::

        P(x) = c_n x^n + \ldots + c_2 x^2 + c_1 x + c_0.

    If *derivative=True* is set, :func:`~mpmath.polyval` simultaneously
    evaluates `P(x)` with the derivative, `P'(x)`, and returns the
    tuple `(P(x), P'(x))`.

        >>> from mpmath import *
        >>> mp.pretty = True
        >>> polyval([3, 0, 2], 0.5)
        2.75
        >>> polyval([3, 0, 2], 0.5, derivative=True)
        (2.75, 3.0)

    The coefficients and the evaluation point may be any combination
    of real or complex numbers.
    """
    if not coeffs:
        return ctx.zero
    p = ctx.convert(coeffs[0])
    q = ctx.zero
    for c in coeffs[1:]:
        if derivative:
            q = p + x*q
        p = c + x*p
    if derivative:
        return p, q
    else:
        return p

@defun
def polyroots(ctx, coeffs, maxsteps=50, cleanup=True, extraprec=10, error=False):
    """
    Computes all roots (real or complex) of a given polynomial. The roots are
    returned as a sorted list, where real roots appear first followed by
    complex conjugate roots as adjacent elements. The polynomial should be
    given as a list of coefficients, in the format used by :func:`~mpmath.polyval`.
    The leading coefficient must be nonzero.

    With *error=True*, :func:`~mpmath.polyroots` returns a tuple *(roots, err)* where
    *err* is an estimate of the maximum error among the computed roots.

    **Examples**

    Finding the three real roots of `x^3 - x^2 - 14x + 24`::

        >>> from mpmath import *
        >>> mp.dps = 15; mp.pretty = True
        >>> nprint(polyroots([1,-1,-14,24]), 4)
        [-4.0, 2.0, 3.0]

    Finding the two complex conjugate roots of `4x^2 + 3x + 2`, with an
    error estimate::

        >>> roots, err = polyroots([4,3,2], error=True)
        >>> for r in roots:
        ...     print(r)
        ...
        (-0.375 + 0.59947894041409j)
        (-0.375 - 0.59947894041409j)
        >>>
        >>> err
        2.22044604925031e-16
        >>>
        >>> polyval([4,3,2], roots[0])
        (2.22044604925031e-16 + 0.0j)
        >>> polyval([4,3,2], roots[1])
        (2.22044604925031e-16 + 0.0j)

    The following example computes all the 5th roots of unity; that is,
    the roots of `x^5 - 1`::

        >>> mp.dps = 20
        >>> for r in polyroots([1, 0, 0, 0, 0, -1]):
        ...     print(r)
        ...
        1.0
        (-0.8090169943749474241 + 0.58778525229247312917j)
        (-0.8090169943749474241 - 0.58778525229247312917j)
        (0.3090169943749474241 + 0.95105651629515357212j)
        (0.3090169943749474241 - 0.95105651629515357212j)

    **Precision and conditioning**

    Provided there are no repeated roots, :func:`~mpmath.polyroots` can typically
    compute all roots of an arbitrary polynomial to high precision::

        >>> mp.dps = 60
        >>> for r in polyroots([1, 0, -10, 0, 1]):
        ...     print r
        ...
        -3.14626436994197234232913506571557044551247712918732870123249
        -0.317837245195782244725757617296174288373133378433432554879127
        0.317837245195782244725757617296174288373133378433432554879127
        3.14626436994197234232913506571557044551247712918732870123249
        >>>
        >>> sqrt(3) + sqrt(2)
        3.14626436994197234232913506571557044551247712918732870123249
        >>> sqrt(3) - sqrt(2)
        0.317837245195782244725757617296174288373133378433432554879127

    **Algorithm**

    :func:`~mpmath.polyroots` implements the Durand-Kerner method [1], which
    uses complex arithmetic to locate all roots simultaneously.
    The Durand-Kerner method can be viewed as approximately performing
    simultaneous Newton iteration for all the roots. In particular,
    the convergence to simple roots is quadratic, just like Newton's
    method.

    Although all roots are internally calculated using complex arithmetic,
    any root found to have an imaginary part smaller than the estimated
    numerical error is truncated to a real number. Real roots are placed
    first in the returned list, sorted by value. The remaining complex
    roots are sorted by real their parts so that conjugate roots end up
    next to each other.

    **References**

    1. http://en.wikipedia.org/wiki/Durand-Kerner_method

    """
    if len(coeffs) <= 1:
        if not coeffs or not coeffs[0]:
            raise ValueError("Input to polyroots must not be the zero polynomial")
        # Constant polynomial with no roots
        return []

    orig = ctx.prec
    weps = +ctx.eps
    try:
        ctx.prec += 10
        tol = ctx.eps * 128
        deg = len(coeffs) - 1
        # Must be monic
        lead = ctx.convert(coeffs[0])
        if lead == 1:
            coeffs = [ctx.convert(c) for c in coeffs]
        else:
            coeffs = [c/lead for c in coeffs]
        f = lambda x: ctx.polyval(coeffs, x)
        roots = [ctx.mpc((0.4+0.9j)**n) for n in xrange(deg)]
        err = [ctx.one for n in xrange(deg)]
        # Durand-Kerner iteration until convergence
        for step in xrange(maxsteps):
            if abs(max(err)) < tol:
                break
            for i in xrange(deg):
                if not abs(err[i]) < tol:
                    p = roots[i]
                    x = f(p)
                    for j in range(deg):
                        if i != j:
                            try:
                                x /= (p-roots[j])
                            except ZeroDivisionError:
                                continue
                    roots[i] = p - x
                    err[i] = abs(x)
        # Remove small imaginary parts
        if cleanup:
            for i in xrange(deg):
                if abs(ctx._im(roots[i])) < weps:
                    roots[i] = roots[i].real
                elif abs(ctx._re(roots[i])) < weps:
                    roots[i] = roots[i].imag * 1j
        roots.sort(key=lambda x: (abs(ctx._im(x)), ctx._re(x)))
    finally:
        ctx.prec = orig
    if error:
        err = max(err)
        err = max(err, ctx.ldexp(1, -orig+1))
        return [+r for r in roots], +err
    else:
        return [+r for r in roots]
